@@ -1,6 +1,12 @@
 use std::collections::HashMap;
 use crate::cache_api::{LoadingCache, CacheLoadingError};
+#[cfg(feature = "ttl-cache")]
+use crate::cache_api::WithMeta;
 use tokio::time::Duration;
+use cache_loader_async_macros::test_with_features;
+use crate::backing::HashMapBacking;
+#[cfg(feature = "ttl-cache")]
+use crate::backing::TtlMeta;
 #[cfg(feature = "lru-cache")]
 use crate::backing::LruCacheBacking;
 #[cfg(feature = "ttl-cache")]
@@ -12,6 +18,7 @@ pub struct ThingOne(u8);
 #[derive(Debug, Clone)]
 pub struct ThingTwo(String);
 
+
 #[tokio::test]
 async fn test_load() {
     let thing_one_static_db: HashMap<String, ThingOne> =
@@ -27,14 +34,14 @@ async fn test_load() {
         .collect();
 
 
-    let (cache_one, _) = LoadingCache::new(move |key: String| {
+    let cache_one = LoadingCache::new(move |key: String| {
         let db_clone = thing_one_static_db.clone();
         async move {
             db_clone.get(&key).cloned().ok_or(1)
         }
     });
 
-    let (cache_two, _) = LoadingCache::new(move |key: String| {
+    let cache_two = LoadingCache::new(move |key: String| {
         let db_clone = thing_two_static_db.clone();
         async move {
             db_clone.get(&key).cloned().ok_or(1)
@@ -51,14 +58,35 @@ async fn test_load() {
     assert_eq!(result_two, "buzz".to_owned());
 }
 
+#[cfg(feature = "ttl-cache")]
 #[tokio::test]
-async fn test_write() {
-    let (cache, _) = LoadingCache::new(move |key: String| {
+async fn test_ttl_load_meta() {
+    let cache: LoadingCache<String, String, u8, _> =
+        LoadingCache::with_meta_loader(TtlCacheBacking::new(Duration::from_secs(1)), move |key: String| {
         async move {
-            Ok(key.to_lowercase())
+            if key.len() < 5 {
+                Ok(key.to_lowercase())
+                    .with_meta(Some(TtlMeta::from(Duration::from_secs(5))))
+            } else {
+                Ok(key.to_lowercase())
+                    .with_meta(None)
+            }
         }
     });
-    let cache: LoadingCache<String, String, u8> = cache;
+
+    assert_eq!(cache.get("a".to_owned()).await.unwrap(), "a".to_owned());
+    assert_eq!(cache.get("bbbbb".to_owned()).await.unwrap(), "bbbbb".to_owned());
+
+    tokio::time::sleep(Duration::from_secs(2)).await;
+
+    assert!(cache.exists("a".to_owned()).await.unwrap());
+    assert!(!cache.exists("bbbbb".to_owned()).await.unwrap());
+}
+
+test_with_features! {
+    test_write cache <String, String, u8> {
+        Ok(key.to_lowercase())
+    }
 
     let lowercase_result = cache.get("LOL".to_owned()).await.unwrap();
     println!("Result of lowercase loader: {}", lowercase_result);
@@ -71,14 +99,10 @@ async fn test_write() {
     assert_eq!(result, "BIG".to_owned());
 }
 
-#[tokio::test]
-async fn test_get_if_present() {
-    let (cache, _) = LoadingCache::new(move |key: String| {
-        async move {
-            Ok(key.to_lowercase())
-        }
-    });
-    let cache: LoadingCache<String, String, u8> = cache;
+test_with_features! {
+    get_if_present cache <String, String, u8> {
+        Ok(key.to_lowercase())
+    }
 
     let option = cache.get_if_present("test".to_owned()).await.unwrap();
     assert!(option.is_none());
@@ -89,14 +113,10 @@ async fn test_get_if_present() {
     assert!(option.is_some());
 }
 
-#[tokio::test]
-async fn test_exists() {
-    let (cache, _) = LoadingCache::new(move |key: String| {
-        async move {
-            Ok(key.to_lowercase())
-        }
-    });
-    let cache: LoadingCache<String, String, u8> = cache;
+test_with_features! {
+    exists cache <String, String, u8> {
+        Ok(key.to_lowercase())
+    }
 
     let exists = cache.exists("test".to_owned()).await.unwrap();
     assert!(!exists);
@@ -109,13 +129,12 @@ async fn test_exists() {
 
 #[tokio::test(flavor = "multi_thread")]
 async fn test_update() {
-    let (cache, _) = LoadingCache::new(move |key: String| {
+    let cache: LoadingCache<String, String, u8, HashMapBacking<_, _>> = LoadingCache::new(move |key: String| {
         async move {
             tokio::time::sleep(Duration::from_millis(500)).await;
             Ok(key.to_lowercase())
         }
     });
-    let cache: LoadingCache<String, String, u8> = cache;
 
     // We test to update an existing key
     cache.set("woob".to_owned(), "Woob".to_owned()).await.ok();
@@ -162,15 +181,39 @@ async fn test_update() {
     assert_eq!(result, "race_condition".to_owned());
 }
 
+test_with_features! {
+    update_if_exists cache <String, String, u8> {
+        tokio::time::sleep(Duration::from_millis(500)).await;
+        Ok(key.to_lowercase())
+    }
+    cache.set("test".to_owned(), "test".to_owned()).await.ok();
+
+    let no_value = cache.update_if_exists("test2".to_owned(), |val| {
+        let mut clone = val.clone();
+        clone.push_str("test");
+        clone
+    }).await.unwrap();
+
+    let two_test = cache.update_if_exists("test".to_owned(), |val| {
+        let mut clone = val.clone();
+        clone.push_str("test");
+        clone
+    }).await.unwrap();
+
+    assert!(no_value.is_none());
+    assert!(two_test.is_some());
+    // returns updated value!
+    assert_eq!(two_test.unwrap(), "testtest");
+}
+
 #[tokio::test(flavor = "multi_thread")]
 async fn test_update_mut() {
-    let (cache, _) = LoadingCache::new(move |key: String| {
+    let cache: LoadingCache<String, String, u8, HashMapBacking<_, _>> = LoadingCache::new(move |key: String| {
         async move {
             tokio::time::sleep(Duration::from_millis(500)).await;
             Ok(key.to_lowercase())
         }
     });
-    let cache: LoadingCache<String, String, u8> = cache;
 
     // We test to update an existing key
     cache.set("woob".to_owned(), "Woob".to_owned()).await.ok();
@@ -211,15 +254,32 @@ async fn test_update_mut() {
     assert_eq!(result, "race_condition".to_owned());
 }
 
-#[tokio::test]
-async fn test_remove() {
-    let (cache, _) = LoadingCache::new(move |key: String| {
-        async move {
-            tokio::time::sleep(Duration::from_millis(500)).await;
-            Ok(key.to_lowercase())
-        }
-    });
-    let cache: LoadingCache<String, String, u8> = cache;
+test_with_features! {
+    update_mut_if_exists cache <String, String, u8> {
+        tokio::time::sleep(Duration::from_millis(500)).await;
+        Ok(key.to_lowercase())
+    }
+    cache.set("test".to_owned(), "test".to_owned()).await.ok();
+
+    let no_value = cache.update_mut_if_exists("test2".to_owned(), |val| {
+        val.push_str("test");
+    }).await.unwrap();
+
+    let two_test = cache.update_mut_if_exists("test".to_owned(), |val| {
+        val.push_str("test");
+    }).await.unwrap();
+
+    assert!(no_value.is_none());
+    assert!(two_test.is_some());
+    // returns updated value!
+    assert_eq!(two_test.unwrap(), "testtest");
+}
+
+test_with_features! {
+    remove cache <String, String, u8> {
+        tokio::time::sleep(Duration::from_millis(500)).await;
+        Ok(key.to_lowercase())
+    }
 
     cache.set("test".to_owned(), "lol".to_owned()).await.ok();
 
@@ -230,14 +290,40 @@ async fn test_remove() {
     assert_eq!(cache.get("test".to_owned()).await.unwrap(), "test".to_owned());
 }
 
-#[tokio::test]
-async fn test_load_error() {
-    let (cache, _) = LoadingCache::new(move |_key: String| {
-        async move {
-            Err(5)
-        }
-    });
-    let cache: LoadingCache<String, String, u8> = cache;
+
+test_with_features! {
+    remove_if cache <u64, u64, u8> {
+        tokio::time::sleep(Duration::from_millis(500)).await;
+        Ok(key * 2)
+    }
+
+    cache.set(1, 2).await.ok();
+    cache.set(2, 4).await.ok();
+    cache.set(3, 6).await.ok();
+    cache.set(4, 8).await.ok();
+    cache.set(5, 10).await.ok();
+
+    assert!(cache.get_if_present(1).await.unwrap().is_some());
+    assert!(cache.get_if_present(2).await.unwrap().is_some());
+    assert!(cache.get_if_present(3).await.unwrap().is_some());
+    assert!(cache.get_if_present(4).await.unwrap().is_some());
+    assert!(cache.get_if_present(5).await.unwrap().is_some());
+
+    cache.remove_if(|(k, _)| k > &3).await.unwrap();
+
+    assert!(cache.get_if_present(1).await.unwrap().is_some());
+    assert!(cache.get_if_present(2).await.unwrap().is_some());
+    assert!(cache.get_if_present(3).await.unwrap().is_some());
+    // next two should be none after remove
+    assert!(cache.get_if_present(4).await.unwrap().is_none());
+    assert!(cache.get_if_present(5).await.unwrap().is_none());
+}
+
+
+test_with_features! {
+    load_error cache <String, String, u8> {
+        Err(5)
+    }
 
     let cache_loading_error = cache.get("test".to_owned()).await.expect_err("Didn't error, what?");
     if let CacheLoadingError::LoadingError(val) = cache_loading_error {
@@ -247,15 +333,11 @@ async fn test_load_error() {
     }
 }
 
-#[tokio::test]
-async fn test_meta() {
-    let (cache, _) = LoadingCache::new(move |key: String| {
-        async move {
-            tokio::time::sleep(Duration::from_millis(500)).await;
-            Ok(key.to_lowercase())
-        }
-    });
-    let cache: LoadingCache<String, String, u8> = cache;
+test_with_features! {
+    meta cache <String, String, u8> {
+        tokio::time::sleep(Duration::from_millis(500)).await;
+        Ok(key.to_lowercase())
+    }
 
     let meta = cache.get_with_meta("key".to_owned()).await.unwrap();
     assert!(!meta.cached);
@@ -267,12 +349,11 @@ async fn test_meta() {
 #[cfg(feature = "lru-cache")]
 #[tokio::test]
 async fn test_lru_backing() {
-    let (cache, _) = LoadingCache::with_backing(LruCacheBacking::new(2), move |key: String| {
+    let cache: LoadingCache<_, _, u8, _> = LoadingCache::with_backing(LruCacheBacking::new(2), move |key: String| {
         async move {
             Ok(key.to_lowercase())
         }
     });
-    let cache: LoadingCache<String, String, u8> = cache;
 
     cache.set("key1".to_owned(), "value1".to_lowercase()).await.ok();
     tokio::time::sleep(Duration::from_secs(1)).await;
@@ -298,13 +379,12 @@ async fn test_lru_backing() {
 #[cfg(feature = "ttl-cache")]
 #[tokio::test]
 async fn test_ttl_backing() {
-    let (cache, _) = LoadingCache::with_backing(
+    let cache: LoadingCache<_, _, u8, _> = LoadingCache::with_backing(
         TtlCacheBacking::new(Duration::from_secs(3)), move |key: String| {
             async move {
                 Ok(key.to_lowercase())
             }
         });
-    let cache: LoadingCache<String, String, u8> = cache;
 
     cache.set("key1".to_owned(), "value1".to_lowercase()).await.ok();
     tokio::time::sleep(Duration::from_secs(2)).await;
@@ -316,4 +396,29 @@ async fn test_ttl_backing() {
     tokio::time::sleep(Duration::from_secs(2)).await;
 
     assert_eq!(cache.exists("key1".to_owned()).await.unwrap(), false);
+}
+
+#[cfg(all(feature = "ttl-cache", feature = "lru-cache"))]
+#[tokio::test]
+async fn test_ttl_lru_backing() {
+    let cache: LoadingCache<String, _, u8, _> = LoadingCache::with_meta_loader(TtlCacheBacking::with_backing(Duration::from_secs(1), LruCacheBacking::new(2)), move |key: String| {
+       async move {
+           if key.len() < 3 {
+               Ok(key.to_lowercase())
+                   .with_meta(Some(TtlMeta::from(Duration::from_secs(15))))
+           } else {
+               Ok(key.to_lowercase())
+                   .with_meta(None)
+           }
+       }
+    });
+    assert_eq!(cache.get("a".to_owned()).await.unwrap(), "a");
+    assert_eq!(cache.get("bbbbb".to_owned()).await.unwrap(), "bbbbb");
+    tokio::time::sleep(Duration::from_secs(2)).await;
+    assert!(cache.exists("a".to_owned()).await.unwrap());
+    assert!(!cache.exists("bbbbb".to_owned()).await.unwrap());
+    assert_eq!(cache.get("bbbbb".to_owned()).await.unwrap(), "bbbbb");
+    assert_eq!(cache.get("ccccc".to_owned()).await.unwrap(), "ccccc");
+    assert_eq!(cache.get("ddddd".to_owned()).await.unwrap(), "ddddd");
+    assert!(!cache.exists("a".to_owned()).await.unwrap());
 }
\ No newline at end of file
